/********************************************************************
 *         This example code is from the book:
 *
 *           Motif Debuggin and Performance Tuning
 *            ISBN 0-13-147984-9
 *         by
 *           Douglas Young
 *           Prentice Hall, 1995
 *
 *         Copyright 1994 by Prentice Hall
 *         All Rights Reserved
 *
 *  Permission to use, copy, modify, and distribute this software for 
 *  any purpose except publication and without fee is hereby granted, provided 
 *  that the above copyright notice appear in all copies of the software.
 * *****************************************************************************/

/********************************************
 * icons.c: Functions for creating icons
 ********************************************/
#include <Xm/Xm.h>
#include <Xm/RowColumn.h>
#include <Xm/LabelG.h>
#include <Xm/PushBG.h>
#include <X11/xpm.h>
#include "icons.h"
#include "globals.h"
extern void SetSelectedWidget(Widget w);
extern void OpenSelected(void);

static Pixmap filePixmap = NULL;
static Pixmap dirPixmap = NULL;

static void SelectCallback ( Widget    w,
                             XtPointer clientData,
                             XtPointer callData)
{
    XmPushButtonCallbackStruct *cbs =
                   ( XmPushButtonCallbackStruct * ) callData;

   /*
    * Always register this icon as the current selected widget.
    */

    SetSelectedWidget ( w );

   /*
    * Open on a double-click
    */

    if ( cbs->click_count > 1 )
        OpenSelected();
}

Pixmap CreatePixmap ( Widget w, FileType type )
{
    int            status;
    Pixmap         pix, mask;
    XpmAttributes  attributes;
    XpmColorSymbol symbols[5];
    Display       *dpy = XtDisplay(w);
    Pixel          bg;

   /*
    * Retrieve the depth and colormap used by this widget
    * and store the results in the corresponding field
    * of an XpmAttributes structure.
    */

    XtVaGetValues ( w,
                    XmNdepth,    &attributes.depth,
                    XmNcolormap, &attributes.colormap,
                    XmNbackground, &bg,
                    NULL );

    symbols[0].name  = "background";
    symbols[0].value = NULL;
    symbols[0].pixel = bg;  

   /*
    * Specify the visual and set the XpmAttributes mask.
    */

    attributes.visual = DefaultVisual ( dpy, 
                                        DefaultScreen ( dpy ) );
    attributes.valuemask =  XpmColorSymbols | XpmDepth |
                            XpmColormap | XpmVisual;

    attributes.colorsymbols = symbols;
    attributes.numsymbols   = 1;

   /*
    * Create the pixmap
    */

    status = XpmCreatePixmapFromData ( dpy,
                    DefaultRootWindow ( dpy ), 
                    type == REGULAR_FILE ? fileIcon : dirIcon,
                    &pix, NULL, &attributes );
    return ( pix );
}


Pixmap GetFilePixmap ( Widget w )
{
    if ( !filePixmap )
        filePixmap = CreatePixmap ( w, REGULAR_FILE );

    return ( filePixmap );
}

Pixmap GetDirPixmap ( Widget w )
{
    if ( !dirPixmap )
        dirPixmap = CreatePixmap ( w, DIRECTORY );

    return ( dirPixmap );
}

static  GC gc = NULL;
static  GC inverseGc = NULL;
static int depth = 0;
 
Widget CreateIcon ( char    *name,
                    Widget   parent,
                    FileType type )
{
    Widget           icon;
    Pixmap           pix;
    XmString         label;
    XmFontList       fontlist;
    Dimension        width, height;
    static XGCValues values;
    int              junk;
    Display         *display = XtDisplay ( parent );
    unsigned char    alignment;
    int              totalWidth, totalHeight;
    Pixmap           labelPixmap;

    if ( type == REGULAR_FILE )
      pix = GetFilePixmap ( parent );
    else
      pix = GetDirPixmap ( parent );
  
    icon = XtVaCreateWidget ( name,
                              xmPushButtonGadgetClass,
                              parent,
                              XmNlabelType, XmPIXMAP,
                              XmNshadowThickness, 0,
                              NULL );
  
    XtAddCallback ( icon, XmNactivateCallback,
                    SelectCallback, NULL );

    if ( !gc )
    {
        /*
         * Create two GCs, one to draw the text and copy 
         * the pixmaps and another that can be used to erase 
         * a pixmap by filling it with the background 
         * color of the label widget. Because the normal GC
         * is used by XmStringDraw, which modifies the font
         * attribute of the GC, allocate this GC using 
         * XtAllocateGC() and specify GCFont as modifiable.
         */

        XtVaGetValues ( parent,   
                        XmNdepth,       &depth,
                        XmNforeground,  &values.foreground, 
                        XmNbackground,  &values.background, 
                        NULL );

        gc = XtAllocateGC ( parent, depth, 
                            GCForeground | GCBackground, 
                            &values, GCFont, 0 );
 
        values.foreground = values.background;
 
        inverseGc = XtGetGC ( parent, 
                              GCForeground | GCBackground,
                              &values );
    }
    
    XtVaGetValues ( icon, 
                    XmNlabelString, &label,
                    XmNfontList,    &fontlist,
                    XmNalignment,   &alignment,
                    NULL );

   /*
    * Compute the size of the label string and the pixmap 
    */
 
    XmStringExtent ( fontlist, label, &width, &height );

    totalWidth = 48 > width ? 48 : width;
    totalHeight = 48 + height + 10;
 
   /*
    * Create the final pixmap using the combined size and 
    * fill the pixmap with the background color of the widget 
    */
 
    labelPixmap = 
      XCreatePixmap ( display, 
		      RootWindowOfScreen ( XtScreen ( parent ) ),
		      totalWidth, totalHeight, depth );
  
    XFillRectangle ( display, labelPixmap, 
                     inverseGc, 0, 0, 
                     totalWidth, totalHeight );
  
   /*
    * Copy the Xpm-created pixmap into the larger pixmap and
    * then draw the string below the pixmap.
    */
  
    XCopyArea ( display, pix, labelPixmap, 
               gc, 0, 0, 48, 48, 
               ( totalWidth - 48 ) / 2, 
               0 );
  
    XmStringDraw ( display, labelPixmap, fontlist, label,
                   gc, 0, 48 + 10, totalWidth, 
                   alignment, XmSTRING_DIRECTION_L_TO_R, NULL );

    /*
     * Install the final pixmap in the widget.
     */
  
     XtVaSetValues ( icon,
                     XmNlabelPixmap, labelPixmap,
                     NULL );
                   
    /*
     * Free the string retrieved from the label widget.
     */

   XmStringFree ( label );
  
   return ( icon );
}

